#!/bin/sh

SCRIPT_CMD=$1
PACKAGE_IDCODE=$2
COMPONENT_FAMILYNAME=$3
COMPONENT_PATH=$4

INSTALL_NAME="install.sh"
UNINSTALL_NAME="uninstall.sh"
UPDATER_NAME="updater"
GENERIC_PACKAGE_INSTALL_NAME="package_install.sh"
GENERIC_PACKAGE_UNINSTALL_NAME="package_uninstall.sh"
DESCRIPT_NAME="header"
LIB_INSTALL_PATH="/usr/local/lib"
BIN_INSTALL_PATH="/usr/local/bin"
HOTPLUG_USB_PATH="/etc/hotplug/usb"

LOCAL_MODULES_DEP_FILE="/opt/local/lib/modules/`uname -r`/modules.dep"

INSTALL_CMD="install"
ACTIVATE_CMD="activate"

install_link()
{
	local SRC_LINK
	local DST_LINK

	SRC_LINK=$1
	DST_LINK=$2
	
	if test -L "$DST_LINK"; then
		cat <<-END_OF_MESSAGE
  		
			The link already exists
		END_OF_MESSAGE
		if test -e "$DST_LINK"; then
			cat <<-END_OF_MESSAGE
  		
				The reference of the link exists
			END_OF_MESSAGE
			# It already exists, look if it's already pointing on us
			if test "$SRC_LINK" != "`readlink $DST_LINK`"; then
				# It's not pointing on us
				echo "Error : The package symbolic link $DST_LINK already exists" 1>&3
				return 1
			else
				cat <<-END_OF_MESSAGE
					The link is pointing on us
				END_OF_MESSAGE
			fi
		else
			cat <<-END_OF_MESSAGE

				The reference of the link doesn't exists anymore -> Replacing it
			END_OF_MESSAGE
			rm -f $DST_LINK
			ln -s $SRC_LINK $DST_LINK
		fi
	else
		if ! ln -s $SRC_LINK $DST_LINK; then
			echo "Error : Can't create $DST_LINK link" 1>&3
			return 1
		fi
	fi

	return 0
}

if test "$SCRIPT_CMD" = $INSTALL_CMD; then
	cat <<-END_OF_MESSAGE

		*************** INSTALL MOD COMPONENT SCRIPT ******************
	END_OF_MESSAGE
elif test "$SCRIPT_CMD" = $ACTIVATE_CMD; then
	cat <<-END_OF_MESSAGE
	   
		*************** ACTIVATE MOD COMPONENT SCRIPT ******************
	END_OF_MESSAGE
else
	echo "Error : Component srcipt unknown command" 1>&3
	exit 1
fi

if ! test -d "$COMPONENT_PATH"; then
	echo "Error : Component path $COMPONENT_PATH doesn't exit" 1>&3
	exit 1
fi

cd $COMPONENT_PATH

if ! $UPDATER_NAME $SCRIPT_CMD $PACKAGE_IDCODE $DESCRIPT_NAME; then
	echo "Error : Couldn't add $PACKAGE_IDCODE in database" 1>&3
	exit 1
fi

# Commands to do for component should be added here

# Descriptor file 3 is custom error flow. It will be printed on the install ticket

# WARNING: DO NOT USE echo command to write on stdout. See TRK 601 !!!!!!!!!!!!!!!!!!!!!!!


if test "$SCRIPT_CMD" = "$ACTIVATE_CMD";then
        ################################################
	# 1st
	#
	# Read the component name from hotplugName
        ################################################
	MODULE_BASENAME=`cat $COMPONENT_PATH/hotplugName`
	MODULE="$MODULE_BASENAME.ko"
	
	cat <<-END_OF_MESSAGE
		Module found is $MODULE	
	END_OF_MESSAGE
	if test ! -e "$COMPONENT_PATH/$MODULE"; then
		echo "Error: Component $COMPONENT_FAMILYNAME doesn't contain module $MODULE" 1>&3
		exit 1
	fi

	if test ! -e "$COMPONENT_PATH/modules.dep"; then
		echo "Error: Component $COMPONENT_FAMILYNAME doesn't contain modules.dep file" 1>&3
		exit 1
	fi

		
        ################################################
	# 2nd 
	#
	# Parse modules.dep to know where to install modules on the system path
        ################################################
	COUNTER=0
	while IFS=: read MOD DEPS; do
		COMPONENT_DEP=${MOD##*/}
		INSTALL_PATH=${MOD%/*}
		cat <<-END_OF_MESSAGE
			Module found in modules.dep: $INSTALL_PATH/$COMPONENT_DEP
		END_OF_MESSAGE
		COUNTER=$(($COUNTER+1))
	done < $COMPONENT_PATH/modules.dep
	
	if test $COUNTER -eq 0; then
		echo "Error: Can't find any matching line in $COMPONENT_PATH/modules.dep"
		exit 1
	fi
	if test $COUNTER -gt 1; then
		echo "Warning: Weird: modules.dep of $COMPONENT_FAMILYNAME have more than one line" 1>&3
	fi
	cat <<-END_OF_MESSAGE

		Result of modules.dep parsing: $MOD:$DEPS
		Base name of module is $INSTALL_PATH/ and $COMPONENT_DEP
	END_OF_MESSAGE
	if test "$COMPONENT_DEP" != "$MODULE"; then
		echo "Error: modules.dep and the module found in $COMPONENT_FAMILYNAME is not matching" 1>&3
		exit 1
	fi

        ################################################
	# 3rd 
	#
	# Create the install path of the module and its link 
        ################################################
	if ! mkdir -p "$INSTALL_PATH"; then
		echo "Error: Can't create path $INSTALL_PATH for component $COMPONENT_FAMILYNAME" 1>&3
		exit 1
	fi
	cat <<-END_OF_MESSAGE
  
		Creating a symbolic link for file $COMPONENT_PATH/$MODULE
	END_OF_MESSAGE

	if ! install_link $COMPONENT_PATH/$MODULE $INSTALL_PATH/$MODULE; then
		exit 1
	fi

        ################################################
	# 4th 
	#
	# Build the modules.dep of the system 
        ################################################
	if ! cp $LOCAL_MODULES_DEP_FILE $LOCAL_MODULES_DEP_FILE.bak; then
		echo "Error: Can't create modules.dep file for component $COMPONENT_FAMILYNAME" 1>&3
		exit 1
	fi

	if ! cat $COMPONENT_PATH/modules.dep >> $LOCAL_MODULES_DEP_FILE.bak; then
		echo "Error: Can't build modules.dep file for component $COMPONENT_FAMILYNAME" 1>&3
		exit 1
	fi

	if ! mv $LOCAL_MODULES_DEP_FILE.bak $LOCAL_MODULES_DEP_FILE; then
		echo "Error: Can't build modules.dep file for component $COMPONENT_FAMILYNAME" 1>&3
		exit 1
	fi

        ################################################
	# 5th 
	#
	# Create hotplug link and copy hotplug usermap file 
        ################################################
	# Create the hotplug link
	if test -e $COMPONENT_PATH/hotplugTarget; then
		HOTPLUG_TARGET=`cat $COMPONENT_PATH/hotplugTarget`
		TARGET_SCRIPT=""
		
		if test ! -e $HOTPLUG_USB_PATH/$HOTPLUG_TARGET; then
			# Let's see if the target script is not contained in the package
			if test ! -e $COMPONENT_PATH/$HOTPLUG_TARGET; then
				echo "Error: The hotplug target script $HOTPLUG_TARGET can't be found" 1>&3
				exit 1
			else
				TARGET_SCRIPT=$COMPONENT_PATH/$HOTPLUG_TARGET
				# Make sure the script embedded in the package is executable
				chown root.root $TARGET_SCRIPT
				chmod 744 $TARGET_SCRIPT
			fi
		else
			TARGET_SCRIPT=$HOTPLUG_USB_PATH/$HOTPLUG_TARGET
		fi
				

		# Create the link for hotplug script
		if ! install_link $TARGET_SCRIPT $HOTPLUG_USB_PATH/$MODULE_BASENAME; then			
			exit 1	
		fi

		# Copy the hotplug usermap
		if ! cp $COMPONENT_PATH/$MODULE_BASENAME.usermap $HOTPLUG_USB_PATH/; then
			echo "Error: Can't copy the $MODULE_BASENAME.usermap in $HOTPLUG_USB_PATH" 1>&3
			exit 1
		fi
	fi
fi



# Exit with 0 for success


exit 0
