#!/bin/sh

SCRIPT_CMD=$1
PACKAGE_IDCODE=$2
COMPONENT_FAMILYNAME=$3
COMPONENT_PATH=$4
FORCE_DELETING=$5

INSTALL_NAME="install.sh"
UNINSTALL_NAME="uninstall.sh"
UPDATER_NAME="updater"
GENERIC_PACKAGE_INSTALL_NAME="package_install.sh"
GENERIC_PACKAGE_UNINSTALL_NAME="package_uninstall.sh"
DESCRIPT_NAME="header"

UNINSTALL_CMD="remove"
UNACTIVATE_CMD="unactivate"

LIB_INSTALL_PATH="/usr/local/lib"
BIN_INSTALL_PATH="/usr/local/bin"
VAR_INSTALL_PATH="/usr/local/var"

HOTPLUG_USB_PATH="/etc/hotplug/usb"
LOCAL_MODULES_DEP_FILE="/opt/local/lib/modules/`uname -r`/modules.dep"

if test "$SCRIPT_CMD" = $UNINSTALL_CMD; then
	cat <<-END_OF_MESSAGE
  
		*************** UNINSTALL MOD COMPONENT SCRIPT ******************
	END_OF_MESSAGE
elif test "$SCRIPT_CMD" = $UNACTIVATE_CMD; then
	cat <<-END_OF_MESSAGE
	
		*************** UNACTIVATE MOD COMPONENT SCRIPT ******************
	END_OF_MESSAGE
else
	echo "Error : Component srcipt unknown command" 1>&3
	exit 1
fi

if ! test -d "$COMPONENT_PATH"; then
	echo "Error Component path $COMPONENT_PATH doesn't exit" 1>&3
	exit 1
fi

cd $COMPONENT_PATH

# Commands to do for component should be added here

# Descriptor file 3 is custom error flow. It will be printed on the install ticket

# WARNING: DO NOT USE echo command to write on stdout. See TRK 601 !!!!!!!!!!!!!!!!!!!!!!!


if test "$SCRIPT_CMD" = $UNACTIVATE_CMD; then
        ################################################
	# 1st
	#
	# Read the component name from hotplugName
        ################################################
	MODULE_BASENAME=`cat $COMPONENT_PATH/hotplugName`
	MODULE="$MODULE_BASENAME.ko"
	MODULE_DEP_LINE=""
	
	cat <<-END_OF_MESSAGE
		Module found is $MODULE	
	END_OF_MESSAGE
	if test ! -e "$COMPONENT_PATH/$MODULE"; then
		echo "Warning: Component $COMPONENT_FAMILYNAME doesn't contain module $MODULE" 1>&3
	fi

	if test ! -e "$COMPONENT_PATH/modules.dep"; then
		echo "Warning: Component $COMPONENT_FAMILYNAME doesn't contain modules.dep file" 1>&3
	fi

        ################################################
	# 2nd 
	#
	# Parse modules.dep to know where the modules is installed on the system
        ################################################
	COUNTER=0
	while IFS=: read MOD DEPS; do
		MODULE_DEP_LINE="$MOD:$DEPS"
		COMPONENT_DEP=${MOD##*/}
		INSTALL_PATH=${MOD%/*}
		cat <<-END_OF_MESSAGE
			Module found in modules.dep: $MODULE_DEP_LINE
		END_OF_MESSAGE
		COUNTER=$(($COUNTER+1))
	done < $COMPONENT_PATH/modules.dep
	
	if test $COUNTER -eq 0; then
		echo "Warning: Can't find any matching line in $COMPONENT_PATH/modules.dep"
	fi
	if test $COUNTER -gt 1; then
		echo "Warning: Weird: modules.dep of $COMPONENT_FAMILYNAME have more than one line" 1>&3
	fi
	if test "$COMPONENT_DEP" != "$MODULE"; then
		echo "Warning: modules.dep and the module found in $COMPONENT_FAMILYNAME is not matching" 1>&3
	fi

        ################################################
	# 3rd 
	#
	# Remove the module and its link 
        ################################################
	if test -e "$INSTALL_PATH/$MODULE"; then
		# It already exists, look if it's already pointing on us
		if test "$COMPONENT_PATH/$MODULE" != "`readlink $INSTALL_PATH/$MODULE`"; then
			# It's not pointing on us
			
			echo "Warning : The package symbolic link $INSTALL_PATH/$MODULE is not pointing on us" 1>&3
		else
			rm -f $INSTALL_PATH/$MODULE
		fi
	fi
	
        ################################################
	# 4th 
	#
	# Rebuild the modules.dep of the system without this modules
        ################################################

	# Read the modules.dep file to remove the dependece of this module
	rm -f $LOCAL_MODULES_DEP_FILE.bak
	COUNTER=0
	while read LINE; do
		if test "$LINE" != "$MODULE_DEP_LINE"; then
			cat >> "$LOCAL_MODULES_DEP_FILE.bak" <<-END_OF_MESSAGE
				$LINE
			END_OF_MESSAGE
		else
			COUNTER=$(($COUNTER+1))
		fi
	done < $LOCAL_MODULES_DEP_FILE

	if test $COUNTER -eq 0; then
		echo "Warning: Can't find dependence to remove for component $COMPONENT_FAMYLINAME" 1>&3
	fi
	if ! mv $LOCAL_MODULES_DEP_FILE.bak $LOCAL_MODULES_DEP_FILE; then
		echo "Warning: Can't build modules.dep file for component $COMPONENT_FAMILYNAME" 1>&3
	fi

        ################################################
	# 5th 
	#
	# Remove hotplug link and delete hotplug usermap file 
        ################################################
	if test -e $COMPONENT_PATH/hotplugTarget; then
		HOTPLUG_TARGET=`cat $COMPONENT_PATH/hotplugTarget`
		
		if test -e $HOTPLUG_USB_PATH/$MODULE_BASENAME; then
			# The link can either point to an existing hotplug script or to a package embedded script
			if test "$HOTPLUG_USB_PATH/$HOTPLUG_TARGET" != "`readlink $HOTPLUG_USB_PATH/$MODULE_BASENAME`"; then
				if test "$COMPONENT_PATH/$HOTPLUG_TARGET" != "`readlink $HOTPLUG_USB_PATH/$MODULE_BASENAME`"; then
					echo "Warning : The package symbolic link $INSTALL_PATH/$COMPONENT is not pointing on us" 1>&3
				else
					rm -f $HOTPLUG_USB_PATH/$MODULE_BASENAME
				fi
			else
				rm -f $HOTPLUG_USB_PATH/$MODULE_BASENAME
			fi
		fi
		rm -f $HOTPLUG_USB_PATH/$MODULE_BASENAME.usermap
	fi
fi

# Exit with 0 for success

if ! $UPDATER_NAME $SCRIPT_CMD $PACKAGE_IDCODE $DESCRIPT_NAME $FORCE_DELETING; then
	echo "Warning : Couldn't $SCRIPT_CMD $PACKAGE_IDCODE from database" 1>&3
fi

exit 0
